!> \file lect-mismip3d_mod.f90
!! Module pour la lecture de topography pour les experiences mismip3d
!<

!> \namespace lect_topo_mismip3
!! Module pour la lecture de topography pour les experiences mismip3d
!! \author Vincent Peyaud, Catherine Ritz
!! \date march 2012

!! @note Used module
!! @note   - use module3D_phy
!<

module lect_topo_mismip3

use module3D_phy
use netcdf
use io_netcdf_grisli
use param_phy_mod

character(len=80)                  :: topo_file        !< netcdf file : initial topography 
character(len=80)                  :: filin
real*8, dimension(:,:),   pointer  :: tab              !< tableau 2d real ecrit dans le fichier

contains

  subroutine input_topo
 
    namelist/topo_mismip3/topo_file

     if (itracebug.eq.1)  call tracebug(' entree dans input topo de MISMIP')


    rewind(num_param)                                  ! pour revenir au debut du fichier param_list.dat
    read(num_param,topo_mismip3)

    write(num_rep_42,428)'!___________________________________________________________' 
    write(num_rep_42,428)'!  module  lect_topo_ant_gen                                '
    write(num_rep_42,topo_mismip3)
    write(num_rep_42,428)'!___________________________________________________________'


    filin     = trim(dirnameinp)//trim(topo_file)
    write(6,*)'filin',filin
    write(6,*)'file_ncdf',file_ncdf


    call Read_Ncdf_var('Bsoc',filin,tab)
    Bsoc(:,:)=tab(:,:)
    Bsoc0(:,:) = Bsoc(:,:)

    call Read_Ncdf_var('S',filin,tab)
    S(:,:)  = tab(:,:)
    S0(:,:) = S(:,:)

    call Read_Ncdf_var('H',filin,tab)
    H(:,:)  = tab(:,:)
    H0(:,:) = H(:,:)

    call Read_Ncdf_var('B',filin,tab)
    B(:,:)=tab(:,:)

!   on met aussi les lectures des vitesses
!   ces variables sont dans 3D donc globales

    call Read_Ncdf_var('Uslid',filin,tab)
    Uxflgz(:,:) = tab(:,:)
    Uyflgz(:,:) = 0.

    call Read_Ncdf_var('Ubar',filin,tab)
    Uxbar(:,:)  = tab(:,:)
    Uybar(:,:)  = 0.

    call strain_rate()  


!   print*, B

428 format(A)


! attribution des variables xmin, ymin, ....
! comme mismip3D est symetrique. La longueur totale est (nx-1)*dx
! ils sont exprimes en km mais l'ecriture netcdf les remet en m

    xmax = nint((nx-1)*dx/2.)/1000.
    ymax = nint((ny-1)*dy/2.)/1000.
    xmin = -xmax
    ymin = -ymax


! modification MISMIP des parametres physiques de param_phy_mod

g        = 9.8                 !<  gravite en m s-2
ro       = 910.0               !<  densities of ice 
secyear  = 3600*24*365         !<  number of s /year
row      = 1028.0              !<  densities of ocean water
rog      = ro*g                !<  ro*g   (glace)
rowg     = row*g               !<  row*g  (ocean) 
romg     = rom*g               !<  rom*g  (asthenosphere)
rofreshg = rofresh*g           !<  rofresh*g 
d00      = ro**3*g**3          !<  ro**3*g**3
dice     = ro/row              !<  ro/row

coef_Sflot = (Row-Ro)/Row      !<   S = coef_Sflot * H + sealevel pour les shelves
coef_Bflot = -Ro/Row           !<   B = coef_Bflot * H + sealevel pour les shelves




  do j=1,ny
     do i=1,nx
        if ((bsoc(i,j)+h(i,j)*ro/row -sealevel).lt.0.) then
           flot(i,j)=.true.
        else
           flot(i,j)=.false.
        endif
     enddo
  enddo

! la glace a le droit d'aller partout
  mk0(:,:) = 1   



! coupure du shelf
!!$  H(305:321,:) = 1.
!!$  S(305:321,:) = 0.1
!!$  B(305:321,:) = S(305:321,:)-H(305:321,:)



  end subroutine input_topo


end module lect_topo_mismip3
